<?php

class ucart extends cmsFrontend {
	
	protected $useOptions = true;

	public function addOrder($buyer){

		$user = cmsUser::getInstance();

		if ($user->isSessionSet('delivery'))
			$delivery = $user->sessionGet('delivery');
		else
			return array('error'=>'Не указан способ доставки');

		$cart_items = $this->model->getCartItems($user->id);
		
		$delivery = $this->model->getDelivery($delivery);

		list($delivery, $cart_items) = cmsEventsManager::hook('before_delivery', array($delivery, $cart_items));

		$summ_items = $items_amount = $this->getSummItems($cart_items);

		$ratio = $this->model->filterEqual('is_enabled', 1)->getRatio(false, 2);

		$ratio = cmsEventsManager::hook('before_ratio_order', $ratio);

		if($ratio) {

			foreach ($ratio as $key=>$value) {
				if( (!empty($value['min_amount']) && $value['min_amount'] > $items_amount) || (!empty($value['max_amount']) && $value['max_amount'] < $items_amount) ){ 
					unset($ratio[$key]);
					continue; 
				}
				if($value['unit'] == 1){
					$summ_items += round($items_amount*($value['value']/100), 2);
				} else {
					$summ_items += $value['value'];
				}
			}

		}

		if($delivery['price']>0 && (empty($delivery['price_free']) || $delivery['price_free'] > $summ_items))
			$summ_items += $delivery['price'];

		$hash = string_random(32);

		$bin_id = !$user->id ? $this->getBinId() : null;

		$order = array(
			'user_id' => $user->id,
			'session_id' => $bin_id,
			'user' => (array)$user,
			'buyer' => $buyer,
			'items' => $cart_items,
			'summ_items' => round($summ_items, 2),
			'ratio' => $ratio,
			'delivery' => $delivery,
			'hash' => $hash

		);

		$order = cmsEventsManager::hook('before_order', $order);

		$result = $this->model->addOrder($order);

		cmsEventsManager::hook('after_order', $result);

		return $hash;

	}

	public function getSummItems($cart_items) {

		$summ=0;

		foreach ($cart_items as $key => $value) {
			$summ += round($value['item']['price']*$value['items_count'], 2);
		}

		return round($summ, 2);

	}

	public function sendOrderForm($items){

        $user = cmsUser::getInstance();

		$fields_form = $this->model->getBuyer();

		$buyer = array();

		foreach ($items as $key => $value) {
			$buyer[$key]['title'] = $fields_form[$key]['title'];
			$buyer[$key]['value'] = $value;
		}

		$result = $this->addOrder($buyer);

		$this->model->clearBin($user->id);

		$this->sendNotify($result);
		
		return $result;

    }

    public function saveBinId() {

    	if (!cmsUser::hasCookie('bin')) {
			cmsUser::setCookiePublic('bin', session_id(), 32850000);
			if (!cmsUser::hasCookie('bin')) {
				if (!cmsUser::isSessionSet('bin')) {
					cmsUser::sessionSet('bin', session_id());
				}
			}
		}

    }

    public function getBinId() {

    	if (cmsUser::hasCookie('bin')) {
			return cmsUser::getCookie('bin');
		} else if (cmsUser::isSessionSet('bin')) {
			return cmsUser::sessionGet('bin');
		} else {
			return false;
		}
    	
    }

    public function saveBinAndOrder($user_id) {

    	$this->model->updateUserBin($user_id);
    	$this->model->updateUserOrder($user_id);

    }

    public function sendNotify($hash){

    	$order = $this->model->getOrderByHash($hash);

    	if (!$order)
    		return false;

    	$order['html'] = cmsTemplate::getInstance()->renderInternal($this, 'order_html', array('item'=>$order));

    	$this->emailNotify($order);
    	$this->idNotify($order);
   		$this->buyerNotify($order);

    }

    public function emailNotify($order) {

    	if(!empty($this->options['email_notify'])) {
    		$messenger = cmsCore::getController('messages');
    		$emails = explode(',', str_replace(' ', '', $this->options['email_notify']));
			foreach ($emails as $email) {
				if(!$this->validate_email($email)) continue;
				$to = array('email' => $email);
		        $letter = array('name' => 'new_admin');
		        $order['html'] = preg_replace('/\r\n|\r|\n/u', '', $order['html']);
				$messenger->sendEmail($to, $letter, array(
		            'order_link' => href_to_abs('ucart', 'order', $order['id']),
		            'order' => $order['html']
		        ));
			}
    	}

    }

    public function idNotify($order) {
    	
    	if($this->options['id_notify']) {
			$ids = explode(',', str_replace(' ', '', $this->options['id_notify']));
			foreach ($ids as $id) {
				if (is_numeric($id)) {
					$notice = '<div>Оформлен новый заказ на сумму: '.$order['summ_items'].' '.$this->options['val'].'</div><div><a href="'.href_to('ucart','order',$order['id']).'">Перейти к заказу</a></div>';
					$this->model->insert('{users}_notices', array('user_id' => $id, 'content' => $notice));
				}
			}
		}

    }

    public function buyerNotify($order) {

    	foreach ($order['buyer'] as $key => $value) {
    		if($value['title'] != 'Email') continue;
    		$order['buyer_email'] = $value['value'];
    		break;
    	}
    	
    	if(empty($order['buyer_email']))
    		return false;

    	if(!$this->validate_email($order['buyer_email']))
			return false;

    	$messenger = cmsCore::getController('messages');
        
        $to = array('email' => $order['buyer_email']);
        $letter = array('name' => 'new_buyer');
        $order['html'] = preg_replace('/\r\n|\r|\n/u', '', $order['html']);
		$messenger->sendEmail($to, $letter, array(
            'ok_link' => href_to_abs('ucart', 'ok', $order['hash']),
            'order' => $order['html']
        ));
    
    }
	
}
